@@ -1,7 +1,9 @@
 import json
 import string
+import logging
 
 import autonetkit.anm
+import autonetkit.nidb
 import autonetkit.log as log
 import autonetkit.plugins
 import autonetkit.plugins.ipv4
@@ -9,44 +11,6 @@ import netaddr
 import networkx as nx
 from networkx.readwrite import json_graph
 
-
-class NidbEncoder(json.JSONEncoder):
-    #TODO: would be better to build this as a graph and then use normal JSON serialization
-    """Recursive handling and formatting for BGP Policy export in JSON format"""
-    def default(self, obj):
-
-        if isinstance(obj, netaddr.IPAddress):
-            return str(obj)
-        if isinstance(obj, netaddr.IPNetwork):
-            return str(obj)
-
-        if isinstance(obj, autonetkit.nidb.NIDB):
-            #TODO: add documentation about serializing anm nodes
-            return {'name': 'nidb',
-                    'children': list(obj.nodes("is_l3device"))}
-
-        if isinstance(obj, autonetkit.nidb.nidb_node):
-            return {'name': obj.id,
-                    'children': list(obj.get_interfaces())}
-            #TODO: also need to return non interface blocks
-# use keys of node
-            return str(obj)
-
-        if isinstance(obj, autonetkit.nidb.overlay_interface):
-            return [{'name': obj.id,
-                'children': list(obj.dict())}]
-        #return stfr(obj)
-
-        if isinstance(obj, dict):
-            return str(obj) #TODO: need to handle as list of children
-            #return [{"name": k, "value": v} for k, v in obj.items
-
-        return json.JSONEncoder.default(self, obj)
-
-def ank_nidb_tree(nidb, indent = 4):
-    data = json.dumps(nidb, cls=NidbEncoder, indent = indent)
-    return data
-
 class AnkEncoder(json.JSONEncoder):
     """Handles netaddr objects by converting to string form"""
     #TODO: look at using skipkeys = True to skip non-basic type keys (can we warn on this?)
@@ -57,6 +21,9 @@ class AnkEncoder(json.JSONEncoder):
             return str(obj)
         if isinstance(obj, netaddr.IPNetwork):
             return str(obj)
+        if isinstance(obj, autonetkit.nidb.nidb_node):
+            #TODO: need to unserialize nidb nodes...
+            return str(obj)
         if isinstance(obj, autonetkit.anm.OverlayNode):
             #TODO: add documentation about serializing anm nodes
             log.warning("%s is anm overlay_node. Use attribute rather than object in compiler." % obj)
@@ -68,13 +35,17 @@ class AnkEncoder(json.JSONEncoder):
             #TODO: add documentation about serializing anm nodes
             log.warning("%s is anm overlay_edge. Use attribute rather than object in compiler." % obj)
             return str(obj)
+        if isinstance(obj, autonetkit.nidb.config_stanza):
+            return str(obj)
         if isinstance(obj, autonetkit.nidb.nidb_node_category):
             #TODO: add documentation about serializing anm nodes
-            log.debug("%s is nidb nidb_node_category. Use attribute rather than object in compiler." % obj)
             return str(obj)
         if isinstance(obj, nx.classes.Graph):
             return json_graph.node_link_data(obj)
 
+        if isinstance(obj, logging.LoggerAdapter):
+            return ""
+
         return json.JSONEncoder.default(self, obj)
 
 def ank_json_dumps(graph, indent = 4):
@@ -225,6 +196,19 @@ def jsonify_anm_with_graphics(anm, nidb = None):
                 })
 
                 #TODO: if no label set, then use node if
+            elif n in phy_graph:
+                #TODO: if in phy but x/y not set, then fall through to setting random
+                #TODO: tidy by moving x/y to higher up before each overlay
+                # try to copy x/y from phy
+                OverlayGraph.node[n].update( {
+                        'x': phy_graph.node[n].get('x'),
+                        'y': phy_graph.node[n].get('y'),
+                        'asn': phy_graph.node[n]['asn'],
+                        'label': phy_graph.node[n]['label'],
+                        'device_type': phy_graph.node[n]['device_type'],
+                        'device_subtype': phy_graph.node[n].get('device_subtype'),
+                        'pop': phy_graph.node[n].get('pop'),
+                })
             else:
                 import random
                 log.debug("Converting to graphics JSON format: node %s not in graphics overlay" % n)
@@ -282,12 +266,20 @@ def jsonify_anm_with_graphics(anm, nidb = None):
 
         border_offset = 20 # so don't plot right at edge
         for n in OverlayGraph:
+            #TODO: do this once per node, rather than each time
             OverlayGraph.node[n]['x'] += - x_min + border_offset
             OverlayGraph.node[n]['y'] += - y_min + border_offset
+            #TODO: make scale to graph size: if size is <100 dont apply
+            #OverlayGraph.node[n]['x'] = round(OverlayGraph.node[n]['x']/25) * 25
+            #OverlayGraph.node[n]['y'] = round(OverlayGraph.node[n]['y']/25) * 25
+
+            # and round to the nearest grid size
+            # for now round to nearest 10
 
         anm_json[overlay_id] = ank_json_dumps(OverlayGraph)
         test_anm_data[overlay_id] = OverlayGraph
 
+
     if nidb:
         test_anm_data['nidb'] = prepare_nidb(nidb)
 
@@ -297,10 +289,11 @@ def jsonify_anm_with_graphics(anm, nidb = None):
 def prepare_nidb(nidb):
     graph = nidb._graph
     for node in graph:
-        graph.node[node]['x'] = graph.node[node]['graphics']['x']
-        graph.node[node]['y'] = graph.node[node]['graphics']['y']
-        graph.node[node]['device_type'] = graph.node[node]['graphics']['device_type']
-        graph.node[node]['device_subtype'] = graph.node[node]['graphics']['device_subtype']
+        if graph.node[node].get("graphics"):
+            graph.node[node]['x'] = graph.node[node]['graphics']['x']
+            graph.node[node]['y'] = graph.node[node]['graphics']['y']
+            graph.node[node]['device_type'] = graph.node[node]['graphics']['device_type']
+            graph.node[node]['device_subtype'] = graph.node[node]['graphics']['device_subtype']
 
         for interface_index in graph.node[node]['_interfaces']:
             try:
