@@ -5,6 +5,7 @@ import itertools
 import pprint
 import string
 import time
+from functools import total_ordering
 
 import autonetkit.log as log
 import networkx as nx
@@ -15,6 +16,13 @@ try:
 except ImportError:
     import pickle
 
+import logging
+
+
+#TODO: rename duplicate use of logger var in log setup
+class CustomAdapter(logging.LoggerAdapter):
+    def process(self, msg, kwargs):
+        return '[%s]: %s' % (self.extra['item'], msg), kwargs
 
 class AutoNetkitException(Exception):
 
@@ -29,7 +37,6 @@ class OverlayNotFound(AutoNetkitException):
     def __str__(self):
         return 'Overlay %s not found' % self.Errors
 
-
 # TODO: rename to OverlayInterface
 
 class overlay_interface(object):
@@ -40,11 +47,15 @@ class overlay_interface(object):
         overlay_id,
         node_id,
         interface_id,
-        ):
+    ):
         object.__setattr__(self, 'anm', anm)
         object.__setattr__(self, 'overlay_id', overlay_id)
         object.__setattr__(self, 'node_id', node_id)
         object.__setattr__(self, 'interface_id', interface_id)
+        logger = logging.getLogger("ANK")
+        logstring = "Node: %s" % str(self)
+        logger = CustomAdapter(logger, {'item': logstring})
+        object.__setattr__(self, 'log', logger)
 
     def __key(self):
         """Note: key doesn't include overlay_id to allow fast cross-layer comparisons"""
@@ -60,14 +71,15 @@ class overlay_interface(object):
 
     def __repr__(self):
         description = self.description or self.interface_id
-        return '(%s, %s)' % (self.node, description)
+        return '(%s, %s)' % (description, self.node)
 
     def __eq__(self, other):
         return self.__key() == other.__key()
 
     def __nonzero__(self):
 
-        # TODO: work out why description and type being set/copied to each overlay
+        # TODO: work out why description and type being set/copied to each
+        # overlay
 
         try:
             interface = self._interface
@@ -81,7 +93,7 @@ class overlay_interface(object):
         # TODO: check how is comparing the nodes
 
         return (self.node, self.interface_id) < (other.node,
-                other.interface_id)
+                                                 other.interface_id)
 
     @property
     def is_bound(self):
@@ -130,17 +142,17 @@ class overlay_interface(object):
 
         if not self.anm.has_overlay(overlay_id):
             log.warning('Trying to access interface %s for non-existent overlay %s'
-                         % (self, overlay_id))
+                        % (self, overlay_id))
             return None
 
         if not self.node_id in self.anm.overlay_nx_graphs[overlay_id]:
             log.debug('Trying to access interface %s for non-existent node %s in overlay %s'
-                       % (self, self.node_id, self.overlay_id))
+                      % (self, self.node_id, self.overlay_id))
             return None
 
         try:
             return overlay_interface(self.anm, overlay_id,
-                    self.node_id, self.interface_id)
+                                     self.node_id, self.interface_id)
         except KeyError:
             return
 
@@ -252,6 +264,7 @@ class overlay_interface(object):
         return [e.dst_int for e in edges]
 
 
+@total_ordering
 class OverlayNode(object):
 
     """OverlayNode"""
@@ -261,16 +274,18 @@ class OverlayNode(object):
         anm,
         overlay_id,
         node_id,
-        ):
+    ):
 
 # Set using this method to bypass __setattr__
 
         object.__setattr__(self, 'anm', anm)
         object.__setattr__(self, 'overlay_id', overlay_id)
-
 # should be able to use _graph from here as anm and overlay_id are defined
-
         object.__setattr__(self, 'node_id', node_id)
+        logger = logging.getLogger("ANK")
+        logstring = "Node: %s" % str(self)
+        logger = CustomAdapter(logger, {'item': logstring})
+        object.__setattr__(self, 'log', logger)
 
     def __hash__(self):
         """"""
@@ -287,11 +302,6 @@ class OverlayNode(object):
 
         return self.interfaces(type='physical')
 
-    def __getnewargs__(self):
-        """"""
-
-        return ()
-
     def __eq__(self, other):
         """"""
 
@@ -300,6 +310,9 @@ class OverlayNode(object):
         except AttributeError:
             return self.node_id == other  # eg compare Node to label
 
+    def __ne__(self, other):
+            return not self.__eq__(other)
+
     @property
     def loopback_zero(self):
         """"""
@@ -339,9 +352,9 @@ class OverlayNode(object):
         else:
             if self_node_string == other_node_string:
                 self_node_id = """""".join([x for x in self.node_id
-                        if x in string.digits])
+                                            if x in string.digits])
                 other_node_id = """""".join([x for x in other.node_id
-                        if x in string.digits])
+                                             if x in string.digits])
                 try:
                     self_node_id = int(self_node_id)
                 except ValueError:
@@ -371,7 +384,7 @@ class OverlayNode(object):
         type='physical',
         *args,
         **kwargs
-        ):
+    ):
         """"""
 
         data = dict(kwargs)
@@ -379,9 +392,10 @@ class OverlayNode(object):
         if self.overlay_id != 'phy' and self.phy:
             next_id = self.phy._next_int_id
             self.phy._interfaces[next_id] = {'type': type,
-                    'description': description}
+                                             'description': description}
 
-            # TODO: fix this workaround for not returning description from phy graph
+            # TODO: fix this workaround for not returning description from phy
+            # graph
 
             data['description'] = description
         else:
@@ -396,7 +410,7 @@ class OverlayNode(object):
         '''Public function to add a loopback interface'''
 
         interface_id = self._add_interface(type='loopback', *args,
-                **kwargs)
+                                           **kwargs)
         return overlay_interface(self.anm, self.overlay_id,
                                  self.node_id, interface_id)
 
@@ -431,7 +445,7 @@ class OverlayNode(object):
         try:
             if key.interface_id in self._interface_ids:
                 return overlay_interface(self.anm, self.overlay_id,
-                        self.node_id, key.interface_id)
+                                         self.node_id, key.interface_id)
         except AttributeError:
 
             # try with key as id
@@ -439,7 +453,7 @@ class OverlayNode(object):
             try:
                 if key in self._interface_ids:
                     return overlay_interface(self.anm, self.overlay_id,
-                            self.node_id, key)
+                                             self.node_id, key)
             except AttributeError:
 
                 # no match for either
@@ -458,12 +472,12 @@ class OverlayNode(object):
             # the interface mappings from phy
 
             return self.phy._graph.node[self.node_id]['_interfaces'
-                    ].keys()
+                                                      ].keys()
         else:
             try:
                 return self._graph.node[self.node_id]['_interfaces']
             except KeyError:
-                log.debug('No interfaces initialised for %s' % self)
+                self.log.debug('No interfaces initialised')
                 return []
 
     @property
@@ -473,7 +487,7 @@ class OverlayNode(object):
         try:
             return self._graph.node[self.node_id]['_interfaces']
         except KeyError:
-            log.debug('No interfaces initialised for %s' % self)
+            self.log.debug('No interfaces initialised for')
             return []
 
     @property
@@ -528,10 +542,10 @@ class OverlayNode(object):
 
             try:
                 return self.anm.overlay_nx_graphs['phy'
-                        ].node[self.node_id]['asn']
+                                                  ].node[self.node_id]['asn']
             except KeyError:
                 if self.node_id not in self.anm.overlay_nx_graphs['phy'
-                        ]:
+                                                                  ]:
                     message = \
                         'Node id %s not found in physical overlay' \
                         % self.node_id
@@ -551,7 +565,7 @@ class OverlayNode(object):
 
         try:
             self.anm.overlay_nx_graphs['phy'].node[self.node_id]['asn'
-                    ] = value
+                                                                 ] = value
         except KeyError:
 
             # set ASN directly on the node, eg for collision domains
@@ -640,9 +654,32 @@ class OverlayNode(object):
         This is useful for accesing attributes passed through from graphml"""
 
         try:
-            return self._graph.node[self.node_id].get(key)
+            node_data = self._graph.node[self.node_id]
         except KeyError:
+            #TODO: only carry out this logic if "Strict mode"
+            if key == "device_type":
+                #TODO: tidy accessors so this doesn't occur, and remove the suppress
+                pass # supress
+                return
+            self.log.debug("Cannot access %s: node not present in %s"
+                %(key, self.overlay_id))
             return
+        else:
+            try:
+                result = node_data[key]
+                return result
+            except KeyError:
+                if key == "device_type":
+                    #TODO: tidy accessors so this doesn't occur, and remove the suppress
+                    pass # supress
+                    return
+
+                # from http://stackoverflow.com/q/2654113
+                self.log.debug("Accessing unset attribute %s in %s" % (key, self.overlay_id))
+                #import inspect
+                #caller = ", ".join(reversed([elem[3] for elem in inspect.stack()[1:-4]]))
+                #self.log.debug("Caller: %s" % caller)
+                return
 
     def get(self, key):
         """For consistency, node.get(key) is neater than getattr(node, key)"""
@@ -655,7 +692,8 @@ class OverlayNode(object):
 
         # TODO: look at mapping the object __dict__ straight to the graph.node[self.node_id]
         # TODO: fix wrt using @x.setter won't work due to following:
-        # as per http://docs.python.org/2/reference/datamodel.html#customizing-attribute-access
+        # as per
+        # http://docs.python.org/2/reference/datamodel.html#customizing-attribute-access
 
         # TODO: fix workaround for asn
 
@@ -675,6 +713,7 @@ class OverlayNode(object):
         return self.__setattr__(key, val)
 
 
+@total_ordering
 class OverlayEdge(object):
 
     """API to access link in network"""
@@ -685,7 +724,7 @@ class OverlayEdge(object):
         overlay_id,
         src_id,
         dst_id,
-        ):
+    ):
 
 # Set using this method to bypass __setattr__
 
@@ -693,6 +732,10 @@ class OverlayEdge(object):
         object.__setattr__(self, 'overlay_id', overlay_id)
         object.__setattr__(self, 'src_id', src_id)
         object.__setattr__(self, 'dst_id', dst_id)
+        logger = logging.getLogger("ANK")
+        logstring = "Interface: %s" % str(self)
+        logger = CustomAdapter(logger, {'item': logstring})
+        object.__setattr__(self, 'log', logger)
 
     def __key(self):
         """Note: key doesn't include overlay_id to allow fast cross-layer comparisons"""
@@ -711,7 +754,7 @@ class OverlayEdge(object):
 
         try:
             return (self.src_id, self.dst_id) == (other.src_id,
-                    other.dst_id)
+                                                  other.dst_id)
         except AttributeError:
             return self.node_id == other
 
@@ -720,11 +763,6 @@ class OverlayEdge(object):
 
         return '%s: (%s, %s)' % (self.overlay_id, self.src, self.dst)
 
-    def __getnewargs__(self):
-        """"""
-
-        return ()
-
     def __getitem__(self, key):
         """"""
 
@@ -892,8 +930,13 @@ class OverlayBase(object):
 
         if overlay_id not in anm.overlay_nx_graphs:
             raise OverlayNotFound(overlay_id)
+            #TODO: return False instead?
         self._anm = anm
         self._overlay_id = overlay_id
+        logger = logging.getLogger("ANK")
+        logstring = "Overlay: %s" % str(overlay_id)
+        logger = CustomAdapter(logger, {'item': logstring})
+        object.__setattr__(self, 'log', logger)
 
     def __repr__(self):
         """"""
@@ -936,7 +979,7 @@ class OverlayBase(object):
             for (src, dst) in self._graph.edges_iter(src_id):
                 if dst == dst_id:
                     return OverlayEdge(self._anm, self._overlay_id,
-                            src, dst)
+                                       src, dst)
 
         # TODO: tidy this logic up
 
@@ -970,7 +1013,7 @@ class OverlayBase(object):
                     # searching by nodes
 
                     return OverlayEdge(self._anm, self._overlay_id,
-                            src, dst)
+                                       src, dst)
             except KeyError:
                 pass  #
 
@@ -1080,8 +1123,8 @@ class OverlayBase(object):
         else:
             data = nodes
         data = sorted(data, key=lambda x: x.get(attribute))
-        for (key, grouping) in itertools.groupby(data, key=lambda x: \
-                x.get(attribute)):
+        for (key, grouping) in itertools.groupby(data, key=lambda x:
+                                                 x.get(attribute)):
             result[key] = list(grouping)
 
         return result
@@ -1091,7 +1134,7 @@ class OverlayBase(object):
         nbunch=None,
         *args,
         **kwargs
-        ):
+    ):
         """"""
 
         if not nbunch:
@@ -1112,7 +1155,7 @@ class OverlayBase(object):
         dst_nbunch=None,
         *args,
         **kwargs
-        ):
+    ):
         """"""
 
 # nbunch may be single node
@@ -1133,7 +1176,7 @@ class OverlayBase(object):
                         kwargs.items())
 
         valid_edges = ((src, dst) for (src, dst) in
-            self._graph.edges_iter(src_nbunch))
+                       self._graph.edges_iter(src_nbunch))
         if dst_nbunch:
             try:
                 dst_nbunch = dst_nbunch.node_id
@@ -1146,10 +1189,11 @@ class OverlayBase(object):
 
                               # only store the id in OverlayEdge
 
-                dst_nbunch = set(dst_nbunch)  # faster membership test than other sequences
+                # faster membership test than other sequences
+                dst_nbunch = set(dst_nbunch)
 
             valid_edges = ((src, dst) for (src, dst) in valid_edges
-                if dst in dst_nbunch)
+                           if dst in dst_nbunch)
 
         if len(args) or len(kwargs):
             all_edges = iter(OverlayEdge(self._anm, self._overlay_id,
@@ -1171,7 +1215,7 @@ class OverlaySubgraph(OverlayBase):
         overlay_id,
         graph,
         name=None,
-        ):
+    ):
         """"""
 
         super(OverlaySubgraph, self).__init__(anm, overlay_id)
@@ -1212,9 +1256,9 @@ class OverlayGraph(OverlayBase):
         retain=None,
         update=False,
         **kwargs
-        ):
+    ):
         """Update won't append data (which could clobber) if node exists"""
-        nbunch = list(nbunch) # listify in case consumed in try/except
+        nbunch = list(nbunch)  # listify in case consumed in try/except
 
         if not retain:
             retain = []
@@ -1241,15 +1285,16 @@ class OverlayGraph(OverlayBase):
             nbunch = add_nodes
         else:
             try:
-                nbunch = [n.node_id for n in nbunch]  # only store the id in overlay
+                # only store the id in overlay
+                nbunch = [n.node_id for n in nbunch]
             except AttributeError:
-                pass # use nbunch directly as the node IDs
+                pass  # use nbunch directly as the node IDs
 
         self._graph.add_nodes_from(nbunch, **kwargs)
         for node in self._graph.nodes():
             node_data = self._graph.node[node]
             if "label" not in node_data:
-                node_data["label"] = str(node) # use node id
+                node_data["label"] = str(node)  # use node id
 
         self._init_interfaces(node_ids)
 
@@ -1258,7 +1303,7 @@ class OverlayGraph(OverlayBase):
         node,
         retain=None,
         **kwargs
-        ):
+    ):
         """Adds node to overlay"""
 
         if not retain:
@@ -1283,11 +1328,11 @@ class OverlayGraph(OverlayBase):
 
     def _init_interfaces(self, nbunch=None):
         """Initialises interfaces"""
-        #TODO: this needs a major refactor!
+        # TODO: this needs a major refactor!
 
-        #store the original bunch to check if going input->phy
+        # store the original bunch to check if going input->phy
         if nbunch is not None:
-            nbunch = list(nbunch) # listify generators
+            nbunch = list(nbunch)  # listify generators
 
         original_nbunch = {}
 
@@ -1308,19 +1353,21 @@ class OverlayGraph(OverlayBase):
 
         phy_graph = self._anm.overlay_nx_graphs['phy']
 
+        initialised_nodes = []
         for node in nbunch:
             try:
                 phy_interfaces = phy_graph.node[node]['_interfaces']
                 interface_data = {'description': None,
                                   'type': 'physical'}
 
-                # need to do dict() to copy, otherwise all point to same memory location -> clobber
+                # need to do dict() to copy, otherwise all point to same memory
+                # location -> clobber
 
                 data = dict((key, dict(interface_data)) for key in
                             phy_interfaces)
                 self._graph.node[node]['_interfaces'] = data
             except KeyError:
-                #TODO: split this off into seperate function
+                # TODO: split this off into seperate function
                 # test if adding from input graph
                 # Note: this has to be done on a node-by-node basis
                 # as ANK allows adding nodes from multiple graphs at once
@@ -1332,19 +1379,26 @@ class OverlayGraph(OverlayBase):
                         if original_node.overlay_id == "input":
                             # are doing input->phy
                             # copy the
-                            original_interfaces = original_node.get("_interfaces")
+                            original_interfaces = original_node.get(
+                                "_interfaces")
                             if original_interfaces is not None:
                                 # Initialise with the keys
                                 int_data = {k: {"description": v.get("description"), "type": v.get("type")}
-                                for k, v in original_interfaces.items()}
-                                self._graph.node[node]['_interfaces'] = int_data
+                                            for k, v in original_interfaces.items()}
+                                self._graph.node[node][
+                                    '_interfaces'] = int_data
 
                 else:
                     # no counterpart in physical graph, initialise
-                    log.debug('Initialise interfaces for %s in %s' % (node,
-                      self._overlay_id))
+                    # Can't do node log becaue node doesn't exist yet
                     self._graph.node[node]['_interfaces'] = \
-                    {0: {'description': 'loopback', 'type': 'loopback'}}
+                        {0: {'description': 'loopback', 'type': 'loopback'}}
+                    initialised_nodes.append(node)
+
+        if len(initialised_nodes):
+            initialised_nodes = [OverlayNode(self.anm, self._overlay_id, n) for n in initialised_nodes]
+            initialised_nodes = sorted([str(n) for n in initialised_nodes])
+            self.log.debug("Initialised interfaces for %s" % ", ".join(initialised_nodes))
 
     def allocate_interfaces(self):
         """allocates edges to interfaces"""
@@ -1355,7 +1409,7 @@ class OverlayGraph(OverlayBase):
                         self.edges()):
                 input_interfaces_allocated = True
             else:
-                log.info('Input interfaces not allocated')
+                log.info('Automatically assigning input interfaces')
                 input_interfaces_allocated = False
 
         if self._overlay_id == 'input':
@@ -1399,9 +1453,9 @@ class OverlayGraph(OverlayBase):
             dst = edge.dst
             dst = edge.dst
             src_int_id = src._add_interface('%s to %s' % (src.label,
-                    dst.label))
+                                                          dst.label))
             dst_int_id = dst._add_interface('%s to %s' % (dst.label,
-                    src.label))
+                                                          src.label))
             edge._interfaces = {}
             edge._interfaces[src.id] = src_int_id
             edge._interfaces[dst.id] = dst_int_id
@@ -1428,7 +1482,7 @@ class OverlayGraph(OverlayBase):
         dst,
         retain=None,
         **kwargs
-        ):
+    ):
         """Adds an edge to the overlay"""
 
         if not retain:
@@ -1460,7 +1514,7 @@ class OverlayGraph(OverlayBase):
         bidirectional=False,
         retain=None,
         **kwargs
-        ):
+    ):
         """Add edges. Unlike NetworkX, can only add an edge if both
         src and dst in graph already.
         If they are not, then they will not be added (silently ignored)
@@ -1485,7 +1539,7 @@ class OverlayGraph(OverlayBase):
                 for edge in ebunch:
                     data = dict((key, edge.get(key)) for key in retain)
                     add_edges.append((edge.src.node_id,
-                            edge.dst.node_id, data))
+                                      edge.dst.node_id, data))
                 ebunch = add_edges
             else:
                 ebunch = [(e.src.node_id, e.dst.node_id, {}) for e in
@@ -1501,16 +1555,16 @@ class OverlayGraph(OverlayBase):
                     _interfaces = {src.node_id: src.interface_id,
                                    dst.node_id: dst.interface_id}
                     ebunch_out.append((src.node_id, dst.node_id,
-                            {'_interfaces': _interfaces}))
+                                       {'_interfaces': _interfaces}))
                 else:
                     try:
                         src_id = src.node_id
                     except AttributeError:
-                        src_id = src # use directly
+                        src_id = src  # use directly
                     try:
                         dst_id = dst.node_id
                     except AttributeError:
-                        dst_id = dst # use directly
+                        dst_id = dst  # use directly
                     ebunch_out.append((src_id, dst_id, {'_interfaces': {}}))
 
             ebunch = ebunch_out
@@ -1528,12 +1582,19 @@ class OverlayGraph(OverlayBase):
 
         if nbunch is None:
             nbunch = self.nodes()
+
+        if nbunch in self:
+            nbunch = [nbunch]  # single node in the list for iteration
+
+        # if the node is in the underlying networkx graph, then map to an overlay node
+        nbunch = [self.node(n) if n in self._graph else n for n in nbunch]
         for node in nbunch:
             for (key, value) in kwargs.items():
                 node.set(key, value)
 
     def update_edges(self, ebunch=None, **kwargs):
         """Sets property defined in kwargs to all edges in ebunch"""
+        # TODO: also allow (src_id, dst_id) and single overlay edge
 
         if not ebunch:
             ebunch = self.edges()
@@ -1564,19 +1625,13 @@ class AbstractNetworkModel(object):
         self.label_attrs = ['label']
         self._build_node_label()
         self.timestamp = time.strftime('%Y%m%d_%H%M%S',
-                time.localtime())
+                                       time.localtime())
 
     def __repr__(self):
         """"""
 
         return 'ANM %s' % self.timestamp
 
-    @staticmethod
-    def __getnewargs__():
-        """"""
-
-        return ()
-
     @property
     def overlay_nx_graphs(self):
         """"""
@@ -1676,7 +1731,7 @@ class AbstractNetworkModel(object):
             'pop',
             'label',
             'asn',
-            ])
+        ])
         return g_in
 
     def add_overlay(
@@ -1687,7 +1742,7 @@ class AbstractNetworkModel(object):
         directed=False,
         multi_edge=False,
         retain=None,
-        ):
+    ):
         """Adds overlay graph of name name"""
 
         if graph:
@@ -1740,9 +1795,9 @@ class AbstractNetworkModel(object):
 
         def custom_label(node):
             return self.label_seperator.join(str(self._overlays['phy'
-                    ].node[node.node_id].get(val)) for val in
-                    self.label_attrs if self._overlays['phy'
-                    ].node[node.node_id].get(val) is not None)
+                                                                ].node[node.node_id].get(val)) for val in
+                                             self.label_attrs if self._overlays['phy'
+                                                                                ].node[node.node_id].get(val) is not None)
 
         self.node_label = custom_label
 
