@@ -18,8 +18,11 @@ SETTINGS = autonetkit.config.settings
 #TODO: note that build network now assumes input graph has interface mappings on nodes/edges
 
 __all__ = ['build']
+from autonetkit.ank_utils import call_log
 
+@call_log
 def load(input_graph_string):
+    #TODO: look at XML header for file type
     try:
         input_graph = graphml.load_graphml(input_graph_string)
     except autonetkit.exception.AnkIncorrectFileFormat:
@@ -29,17 +32,19 @@ def load(input_graph_string):
         except ImportError, e:
             log.debug("Unable to load autonetkit_cisco %s" % e)
             return  # module not present (development module)
-        input_graph = cisco_load.load(input_graph_string)
-# add local deployment host
-        SETTINGS['General']['deploy'] = True
-        SETTINGS['Deploy Hosts']['internal'] = {
-            'cisco': {
-            'deploy': True,
-            },
-        }
+        else:
+            input_graph = cisco_load.load(input_graph_string)
+            # add local deployment host
+            SETTINGS['General']['deploy'] = True
+            SETTINGS['Deploy Hosts']['internal'] = {
+                        'VIRL': {
+                        'deploy': True,
+                        },
+                        }
 
     return input_graph
 
+@call_log
 def grid_2d(dim):
     """Creates a 2d grid of dimension dim"""
     graph = nx.grid_2d_graph(dim, dim)
@@ -70,6 +75,7 @@ def grid_2d(dim):
 
     return graph
 
+@call_log
 def initialise(input_graph):
     """Initialises the input graph with from a NetworkX graph"""
     anm = autonetkit.anm.AbstractNetworkModel()
@@ -82,14 +88,14 @@ def initialise(input_graph):
         # if not specified then automatically assign interface names
         g_in.data.specified_int_names = False
 
-    import autonetkit.plugins.graph_product as graph_product
-    graph_product.expand(g_in)  # apply graph products if relevant
+    #import autonetkit.plugins.graph_product as graph_product
+    #graph_product.expand(g_in)  # apply graph products if relevant
 
     expand_fqdn = False
     # TODO: make this set from config and also in the input file
     if expand_fqdn and len(ank_utils.unique_attr(g_in, "asn")) > 1:
         # Multiple ASNs set, use label format device.asn
-        anm.set_node_label(".", ['label', 'pop', 'asn'])
+        anm.set_node_label(".", ['label', 'asn'])
 
     g_in.update(
         g_in.nodes("is_router", platform="junosphere"), syntax="junos")
@@ -97,22 +103,27 @@ def initialise(input_graph):
     g_in.update(g_in.nodes("is_router", platform="netkit"), syntax="quagga")
     g_in.update(g_in.nodes("is_server", platform="netkit"), syntax="quagga")
 
+    autonetkit.ank.set_node_default(g_in,  specified_int_names=None)
+
     g_graphics = anm.add_overlay("graphics")  # plotting data
     g_graphics.add_nodes_from(g_in, retain=['x', 'y', 'device_type',
-        'label', 'device_subtype', 'pop', 'asn'])
+        'label', 'device_subtype', 'asn'])
 
-    if g_in.data.Creator == "Maestro":
+    if g_in.data.Creator == "VIRL":
         #TODO: move this to other module
         # Multiple ASNs set, use label format device.asn
-        anm.set_node_label(".", ['label_full'])
+        #anm.set_node_label(".", ['label_full'])
+        pass
 
     return anm
 
+@call_log
 def check_server_asns(anm):
     """Checks that servers have appropriate ASN allocated.
     Warns and auto-corrects servers which are connected to routers of a difference AS.
     #TODO: provide manual over-ride for this auto-correct.
     """
+    #TODO: Move to validate module?
     g_phy = anm['phy']
 
     for server in g_phy.nodes('is_server'):
@@ -123,7 +134,7 @@ def check_server_asns(anm):
         if server.asn not in l3_neighbor_asns:
             neighs_with_asn = ["%s: AS %s" % (n, n.asn)
                 for n in l3_neighbors] # tuples for warning message
-            log.warning("Server %s does not belong to same ASN as neighbors %s" % (server, neighs_with_asn))
+            server.log.warning("Server does not belong to same ASN as neighbors %s" % (neighs_with_asn))
 
             if len(l3_neighbors) == 1:
                 # single ASN of neighbor -> auto correct
@@ -136,6 +147,7 @@ def check_server_asns(anm):
                         (server, server.asn))
 
 
+@call_log
 def apply_design_rules(anm):
     """Applies appropriate design rules to ANM"""
     g_in = anm['input']
@@ -149,6 +161,7 @@ def apply_design_rules(anm):
     check_server_asns(anm)
     #autonetkit.update_http(anm)
 
+    from autonetkit.design.mpls import build_vrf
     build_vrf(anm) # need to do before to add loopbacks before ip allocations
     from autonetkit.design.ip import build_ip, build_ipv4, build_ipv6
     build_ip(anm) # ip infrastructure topology
@@ -185,9 +198,7 @@ def apply_design_rules(anm):
         anm.add_overlay("ipv6") # placeholder for compiler logic
 
     default_igp = g_in.data.igp or "ospf"
-    non_igp_nodes = [n for n in g_in if not n.igp]
-#TODO: should this be modifying g_in?
-    g_in.update(non_igp_nodes, igp=default_igp) # store igp onto each node
+    ank_utils.set_node_default(g_in,  igp=default_igp)
 
     ank_utils.copy_attr_from(g_in, g_phy, "include_csr")
 
@@ -213,6 +224,7 @@ def apply_design_rules(anm):
 
 # post-processing
     if anm['phy'].data.enable_routing:
+        from autonetkit.design.mpls import mark_ebgp_vrf, build_ibgp_vpn_v4
         mark_ebgp_vrf(anm)
         build_ibgp_vpn_v4(anm) # build after bgp as is based on
     #autonetkit.update_http(anm)
@@ -227,250 +239,59 @@ def apply_design_rules(anm):
     return anm
 
 
+@call_log
 def build(input_graph):
     """Main function to build network overlay topologies"""
+    anm = None
     try:
         anm = initialise(input_graph)
         anm = apply_design_rules(anm)
         #print {str(node): {'x': node.x, 'y': node.y} for node in anm['input']}
+        import autonetkit
+        autonetkit.update_http(anm)
     except Exception, e:
         # Send the visualisation to help debugging
         import autonetkit
-        autonetkit.update_http(anm)
-        raise
+        try:
+            autonetkit.update_http(anm)
+        except Exception, e:
+            # problem with vis -> could be coupled with original exception - raise original
+            log.warning("Unable to visualise: %s" % e)
+        raise # raise the original exception
     return anm
 
-def vrf_pre_process(anm):
-    """Marks nodes in g_in as appropriate based on vrf roles.
-    CE nodes -> ibgp_level = 0, so not in ibgp (this is allocated later)
-    """
-    log.debug("Applying VRF pre-processing")
-    g_vrf = anm['vrf']
-    for node in g_vrf.nodes(vrf_role = "CE"):
-        log.debug("Marking CE node %s as non-ibgp" % node)
-        node['input'].ibgp_level = 0
 
-def allocate_vrf_roles(g_vrf):
-    """Allocate VRF roles"""
-    g_phy = g_vrf.anm['phy']
-    for node in g_vrf.nodes(vrf_role="CE"):
-        if not node.vrf:
-            node.vrf = "default_vrf"
+def remove_parallel_switch_links(anm):
+    g_phy = anm['phy']
+    subs = ank_utils.connected_subgraphs(g_phy, g_phy.nodes('is_switch'))
+    for component in subs:
+        log.debug("Checking for multiple links to switch cluster %s" % str(sorted(component)))
 
-    for node in g_vrf.nodes('vrf'):
-        node.vrf_role = "CE"
+        # Collect all links into this cluster
+        external_edges = []
+        for switch in component:
+            for edge in switch.edges():
+                if edge.dst not in component:
+                    external_edges.append(edge)
 
-    non_ce_nodes = [node for node in g_vrf if node.vrf_role != "CE"]
+        # Group by the node they link to
+        from collections import defaultdict
+        check_dict = defaultdict(list)
+        for edge in external_edges:
+            check_dict[edge.dst].append(edge)
 
-    for node in non_ce_nodes:
-        phy_neighbors = g_phy.node(node).neighbors("is_router")
-        # neighbors from physical graph for connectivity
-        phy_neighbors = [neigh for neigh in phy_neighbors]
-            # filter to just this asn
-        if any(g_vrf.node(neigh).vrf_role == "CE" for neigh in phy_neighbors):
-            # phy neigh has vrf set in this graph
-            node.vrf_role = "PE"
-        else:
-            node.vrf_role = "P"  # default role
+        # Check to see if any nodes have more than one link into this aggregate
+        for dst, edges in check_dict.items():
+            if len(edges) > 1:
+                edges_to_remove = sorted(edges)[1:] # remove all but first
+                interfaces = ", ".join(sorted(str(edge.dst_int['phy']) for edge in edges))
+                interfaces_to_disconnect = ", ".join(sorted(str(edge.dst_int['phy']) for edge in edges_to_remove))
+                dst.log.warning("Multiple edges exist to same switch cluster: %s (%s). Removing edges from interfaces %s" % (
+                    str(sorted(component)), interfaces, interfaces_to_disconnect))
 
-def add_vrf_loopbacks(g_vrf):
-    """Adds loopbacks for VRFs, and stores VRFs connected to PE router"""
-    #autonetkit.update_http(anm)
-    for node in g_vrf.nodes(vrf_role="PE"):
-        node_vrf_names = {n.vrf for n in node.neighbors(vrf_role="CE")}
-        node.node_vrf_names = node_vrf_names
-        node.rd_indices = {}
-        for index, vrf_name in enumerate(node_vrf_names, 1):
-            node.rd_indices[vrf_name] = index
-            node.add_loopback(vrf_name=vrf_name,
-                              description="loopback for vrf %s" % vrf_name)
-
-def build_ibgp_vpn_v4(anm):
-    """Based on the ibgp_v4 hierarchy rules.
-    Exceptions:
-    1. Remove links to (PE, RRC) nodes
-
-    CE nodes are excluded from RR hierarchy ibgp creation through pre-process step
-
-    """
-    #TODO: remove the bgp layer and have just ibgp and ebgp
-    # TODO: build from design rules, currently just builds from ibgp links in bgp layer
-    g_bgp = anm['bgp']
-    g_ibgp_v4 = anm['ibgp_v4']
-    g_vrf = anm['vrf']
-    g_ibgp_vpn_v4 = anm.add_overlay("ibgp_vpn_v4", directed=True)
-
-    ibgp_v4_nodes = list(g_ibgp_v4.nodes())
-    pe_nodes = set(g_vrf.nodes(vrf_role = "PE"))
-    pe_rrc_nodes = {n for n in ibgp_v4_nodes if
-            n in pe_nodes and n.ibgp_role == "RRC"}
-    ce_nodes = set(g_vrf.nodes(vrf_role = "CE"))
-
-    if len(pe_nodes) == len(ce_nodes) == len(pe_rrc_nodes) == 0:
-        # no vrf nodes to connect
-        return
-
-    #TODO: extend this to only connect nodes which are connected in VRFs, so don't set to others
-
-    ibgp_vpn_v4_nodes = (n for n in ibgp_v4_nodes
-            if n not in pe_rrc_nodes and n not in ce_nodes)
-    g_ibgp_vpn_v4.add_nodes_from(ibgp_vpn_v4_nodes, retain = "ibgp_level")
-    g_ibgp_vpn_v4.add_edges_from(g_ibgp_v4.edges(), retain = "direction")
-
-    for node in g_ibgp_vpn_v4:
-        if node.ibgp_level in (2, 3): # HRR or RR
-            node.retain_route_target = True
-
-    ce_edges = [e for e in g_ibgp_vpn_v4.edges()
-            if e.src in ce_nodes or e.dst in ce_nodes]
-
-    # mark ibgp direction
-    ce_pe_edges = []
-    pe_ce_edges = []
-    for edge in g_ibgp_vpn_v4.edges():
-        if (edge.src.vrf_role, edge.dst.vrf_role) == ("CE", "PE"):
-            edge.direction = "up"
-            edge.vrf = edge.src.vrf
-            ce_pe_edges.append(edge)
-        elif (edge.src.vrf_role, edge.dst.vrf_role) == ("PE", "CE"):
-            edge.direction = "down"
-            edge.vrf = edge.dst.vrf
-            pe_ce_edges.append(edge)
-
-    #TODO: Document this
-    g_ibgpv4 = anm['ibgp_v4']
-    g_ibgpv6 = anm['ibgp_v6']
-    g_ibgpv4.remove_edges_from(ce_edges)
-    g_ibgpv6.remove_edges_from(ce_edges)
-    g_ibgpv4.add_edges_from(ce_pe_edges, retain = ["direction", "vrf"])
-    g_ibgpv4.add_edges_from(pe_ce_edges, retain = ["direction", "vrf"])
-    g_ibgpv6.add_edges_from(ce_pe_edges, retain = ["direction", "vrf"])
-    g_ibgpv6.add_edges_from(pe_ce_edges, retain = ["direction", "vrf"])
-    for edge in pe_ce_edges:
-        # mark as exclude so don't include in standard ibgp config stanzas
-        if g_ibgpv4.has_edge(edge):
-            edge['ibgp_v4'].exclude = True
-        if g_ibgpv6.has_edge(edge):
-            edge['ibgp_v6'].exclude = True
-
-# legacy
-    g_bgp = anm['bgp']
-    g_bgp.remove_edges_from(ce_edges)
-    g_bgp.add_edges_from(ce_pe_edges, retain = ["direction", "vrf", "type"])
-    g_bgp.add_edges_from(pe_ce_edges, retain = ["direction", "vrf", "type"])
-
-    # also need to modify the ibgp_v4 and ibgp_v6 graphs
-
-def build_mpls_ldp(anm):
-    """Builds MPLS LDP"""
-    g_in = anm['input']
-    g_vrf = anm['vrf']
-    g_l3conn = anm['l3_conn']
-    g_mpls_ldp = anm.add_overlay("mpls_ldp")
-    nodes_to_add = [n for n in g_in.nodes("is_router")
-            if n['vrf'].vrf_role in ("PE", "P")]
-    g_mpls_ldp.add_nodes_from(nodes_to_add, retain=["vrf_role", "vrf"])
-
-    # store as set for faster lookup
-    pe_nodes = set(g_vrf.nodes(vrf_role = "PE"))
-    p_nodes = set(g_vrf.nodes(vrf_role = "P"))
-
-    pe_to_pe_edges = (e for e in g_l3conn.edges()
-            if e.src in pe_nodes and e.dst in pe_nodes)
-    g_mpls_ldp.add_edges_from(pe_to_pe_edges)
-
-    pe_to_p_edges = (e for e in g_l3conn.edges()
-            if e.src in pe_nodes and e.dst in p_nodes
-            or e.src in p_nodes and e.dst in pe_nodes)
-    g_mpls_ldp.add_edges_from(pe_to_p_edges)
-
-    p_to_p_edges = (e for e in g_l3conn.edges()
-            if e.src in p_nodes and e.dst in p_nodes)
-    g_mpls_ldp.add_edges_from(p_to_p_edges)
-
-def mark_ebgp_vrf(anm):
-    g_ebgp = anm['ebgp']
-    g_vrf = anm['vrf']
-    g_ebgpv4 = anm['ebgp_v4']
-    g_ebgpv6 = anm['ebgp_v6']
-    pe_nodes = set(g_vrf.nodes(vrf_role = "PE"))
-    ce_nodes = set(g_vrf.nodes(vrf_role = "CE"))
-    for edge in g_ebgpv4.edges():
-        if (edge.src in pe_nodes and edge.dst in ce_nodes):
-            # exclude from "regular" ebgp (as put into vrf stanza)
-            edge.exclude = True
-            edge.vrf = edge.dst['vrf'].vrf
-
-    for edge in g_ebgpv6.edges():
-        if (edge.src in pe_nodes and edge.dst in ce_nodes):
-             # exclude from "regular" ebgp (as put into vrf stanza)
-            edge.exclude = True
-            edge.vrf = edge.dst['vrf'].vrf
-
-def build_vrf(anm):
-    """Build VRF Overlay"""
-    g_in = anm['input']
-    g_l3conn = anm['l3_conn']
-    g_vrf = anm.add_overlay("vrf")
-
-    if not any(True for n in g_in if n.is_router and n.vrf):
-        log.debug("No VRFs set")
-        return
-
-    g_vrf.add_nodes_from(g_in.nodes("is_router"), retain=["vrf_role", "vrf"])
-
-    allocate_vrf_roles(g_vrf)
-
-    vrf_pre_process(anm)
-
-    def is_pe_ce_edge(edge):
-        src_vrf_role = g_vrf.node(edge.src).vrf_role
-        dst_vrf_role = g_vrf.node(edge.dst).vrf_role
-        return (src_vrf_role, dst_vrf_role) in (("PE", "CE"), ("CE", "PE"))
-
-    vrf_add_edges = (e for e in g_l3conn.edges()
-           if is_pe_ce_edge(e))
-    #TODO: should mark as being towards PE or CE
-    g_vrf.add_edges_from(vrf_add_edges)
-
-    def is_pe_p_edge(edge):
-        src_vrf_role = g_vrf.node(edge.src).vrf_role
-        dst_vrf_role = g_vrf.node(edge.dst).vrf_role
-        return (src_vrf_role, dst_vrf_role) in (("PE", "P"), ("P", "PE"))
-    vrf_add_edges = (e for e in g_l3conn.edges()
-            if is_pe_p_edge(e))
-    g_vrf.add_edges_from(vrf_add_edges)
-
-    build_mpls_ldp(anm)
-    # add PE to P edges
-
-    add_vrf_loopbacks(g_vrf)
-    # allocate route-targets per AS
-    # This could later look at connected components for each ASN
-    route_targets = {}
-    for asn, devices in ank_utils.groupby("asn", g_vrf.nodes(vrf_role = "PE")):
-        asn_vrfs = [d.node_vrf_names for d in devices]
-        # flatten list to unique set
-        asn_vrfs = set(itertools.chain.from_iterable(asn_vrfs))
-        route_targets[asn] = {vrf: "%s:%s" % (asn, index)
-                for index, vrf in enumerate(sorted(asn_vrfs), 1)}
-
-    g_vrf.data.route_targets = route_targets
-
-    for node in g_vrf:
-        vrf_loopbacks = node.interfaces("is_loopback", "vrf_name")
-        for index, interface in enumerate(vrf_loopbacks, start = 101):
-            interface.index = index
-
-    for edge in g_vrf.edges():
-        # Set the vrf of the edge to be that of the CE device (either src or dst)
-        edge.vrf = edge.src.vrf if edge.src.vrf_role is "CE" else edge.dst.vrf
-
-    # map attributes to interfaces
-    for edge in g_vrf.edges():
-        for interface in edge.interfaces():
-            interface.vrf_name = edge.vrf
+                g_phy.remove_edges_from(edges_to_remove)
 
+@call_log
 def build_phy(anm):
     """Build physical overlay"""
     g_in = anm['input']
@@ -486,10 +307,11 @@ def build_phy(anm):
     if g_in.data.Creator == "Topology Zoo Toolset":
         ank_utils.copy_attr_from(g_in, g_phy, "Network")
 
+    ank_utils.set_node_default(g_phy,  Network=None)
     g_phy.add_edges_from(g_in.edges(type="physical"))
     # TODO: make this automatic if adding to the physical graph?
 
-    if g_in.data.Creator == "Maestro":
+    if g_in.data.Creator == "VIRL":
         g_phy.data.mgmt_interfaces_enabled = g_in.data.mgmt_interfaces_enabled
         #TODO: remove this code now allocated externally
         g_phy.data.mgmt_address_start = g_in.data.mgmt_address_start
@@ -505,6 +327,8 @@ def build_phy(anm):
         ank_utils.copy_attr_from(g_in, g_phy, "server_username")
         ank_utils.copy_attr_from(g_in, g_phy, "server_ssh_key")
 
+    ank_utils.set_node_default(g_phy,  use_ipv4 = False, use_ipv6=False)
+
     g_phy.allocate_interfaces()
 
     for node in g_phy:
@@ -513,6 +337,9 @@ def build_phy(anm):
             if specified_id:
                 interface.specified_id = specified_id # map across
 
+    remove_parallel_switch_links(anm)
+
+@call_log
 def build_l3_connectivity(anm):
     """ creates l3_connectivity graph, which is switch nodes aggregated and exploded"""
     #TODO: use this as base for ospf, ebgp, ip, etc rather than exploding in each
@@ -532,6 +359,7 @@ def build_l3_connectivity(anm):
         edge.src_int.multipoint = True
         edge.dst_int.multipoint = True
 
+@call_log
 def build_conn(anm):
     """Build connectivity overlay"""
     g_in = anm['input']
