@@ -7,7 +7,7 @@ import netaddr
 from autonetkit.ank import sn_preflen_to_network
 from autonetkit.compiler import sort_sessions
 from autonetkit.compilers.device.router_base import RouterCompiler
-
+from autonetkit.nidb import config_stanza
 
 class IosBaseCompiler(RouterCompiler):
 
@@ -35,6 +35,7 @@ class IosBaseCompiler(RouterCompiler):
         node.use_cdp = phy_node.use_cdp
 
         if node in self.anm['ospf']:
+            node.add_stanza("ospf")
             node.ospf.use_ipv4 = phy_node.use_ipv4
             node.ospf.use_ipv6 = phy_node.use_ipv6
 
@@ -68,6 +69,7 @@ class IosBaseCompiler(RouterCompiler):
                     ipv4_loopback_subnet.prefixlen)
 
         if node.ip.use_ipv6:
+            #TODO: clean this up so can set on router_base: call cidr not address and update templates
             node.loopback_zero.use_ipv6 = True
             ipv6_loopback_zero = phy_loopback_zero['ipv6']
             node.loopback_zero.ipv6_address = \
@@ -585,29 +587,6 @@ class IosXrCompiler(IosBaseCompiler):
     def ospf(self, node):
         super(IosXrCompiler, self).ospf(node)
 
-        g_ospf = self.anm['ospf']
-        interfaces_by_area = defaultdict(list)
-
-        for interface in node.physical_interfaces:
-            if interface.exclude_igp:
-                continue  # don't configure IGP for this interface
-
-            ospf_int = g_ospf.interface(interface)
-            if ospf_int and ospf_int.is_bound:
-                area = ospf_int.area
-                area = str(area)  # can't serialize IPAddress object to JSON
-                interfaces_by_area[area].append({'id': interface.id,
-                        'cost': int(ospf_int.cost), 'passive': False})
-
-        loopback_zero = node.loopback_zero
-        ospf_loopback_zero = g_ospf.interface(loopback_zero)
-        router_area = ospf_loopback_zero.area  # area assigned to router
-        router_area = str(router_area)  # can't serialize IPAddress object to JSON
-        interfaces_by_area[router_area].append({'id': node.loopback_zero.id,
-                'cost': 0, 'passive': True})
-
-        node.ospf.interfaces = dict(interfaces_by_area)
-
     def eigrp(self, node):
         super(IosXrCompiler, self).eigrp(node)
 
@@ -665,6 +644,34 @@ class IosXrCompiler(IosBaseCompiler):
 
 class NxOsCompiler(IosBaseCompiler):
 
+    def compile(self, node):
+        super(NxOsCompiler, self).compile(node)
+        self.mpls_te(node)
+        self.mpls_oam(node)
+
+    def mpls_te(self, node):
+        g_mpls_te = self.anm['mpls_te']
+        if node not in g_mpls_te:
+            return   # no mpls te configured
+
+        if node.supported_features.mpls_te is False:
+            node.log.warning("Feature MPLS TE is not supported for %s on the %s platform" % (node.device_subtype, node.platform))
+
+    def mpls_oam(self, node):
+        g_mpls_oam = self.anm['mpls_oam']
+        if node not in g_mpls_oam:
+            return   # no mpls oam configured
+
+        if node.supported_features.mpls_oam is False:
+            node.log.warning("Feature MPLS OAM is not supported for %s the on %s platform" % (node.device_subtype, node.platform))
+
+    def vrf(self, node):
+        g_vrf = self.anm['vrf']
+        if node not in g_vrf:
+            return   # no mpls oam configured
+        if node.supported_features.vrf is False:
+            node.log.warning("Feature VRF is not supported for %s on the %s platform" % (node.device_subtype, node.platform))
+
     def interfaces(self, node):
 
 # need to aggregate areas
