@@ -2,6 +2,7 @@
 # -*- coding: utf-8 -*-
 from autonetkit.compiler import sort_sessions
 from autonetkit.compilers.device.device_base import DeviceCompiler
+from autonetkit.nidb import config_stanza
 
 import autonetkit.plugins.naming as naming
 import autonetkit.log as log
@@ -78,14 +79,15 @@ class RouterCompiler(DeviceCompiler):
         super(RouterCompiler, self).__init__(nidb, anm)
 
     def compile(self, node):
+        node.do_render = True # turn on rendering
+
         phy_node = self.anm['phy'].node(node)
         ipv4_node = self.anm['ipv4'].node(node)
 
         node.ip.use_ipv4 = phy_node.use_ipv4 or False
         node.ip.use_ipv6 = phy_node.use_ipv6 or False
         if not (node.ip.use_ipv4 and node.ip.use_ipv6):
-            log.debug('Neither IPv4 nor IPv6 specified for %s, using IPv4'
-                       % node)
+            node.log.debug('Neither IPv4 nor IPv6 specified: using default IPv4')
 
             # node.ip.use_ipv4 = True
 
@@ -106,7 +108,14 @@ class RouterCompiler(DeviceCompiler):
             self.isis(node)
         if self.anm.has_overlay('eigrp') and node in self.anm['eigrp']:
             self.eigrp(node)
-        if self.anm.has_overlay('bgp') and node in self.anm['bgp']:
+        #TODO: allow this to work for ibgp and ebgp
+        if (
+            (self.anm.has_overlay('ebgp_v4') and node in self.anm['ebgp_v4'])
+            or (self.anm.has_overlay('ibgp_v4') and node in self.anm['ibgp_v4'])
+            or (self.anm.has_overlay('ebgp_v6') and node in self.anm['ebgp_v6'])
+            or (self.anm.has_overlay('ibgp_v6') and node in self.anm['ibgp_v6'])
+            or
+            (self.anm.has_overlay('bgp') and node in self.anm['bgp'])):
             self.bgp(node)
 
     def interfaces(self, node):
@@ -115,6 +124,17 @@ class RouterCompiler(DeviceCompiler):
         node.loopback_zero.id = self.lo_interface
         node.loopback_zero.description = 'Loopback'
 
+        if node.ip.use_ipv4:
+            ipv4_node = self.anm['ipv4'].node(node)
+            node.loopback_zero.ipv4_address = ipv4_node.loopback
+            node.loopback_zero.ipv4_subnet = node.loopback_subnet
+
+        #TODO: bne consistent wit hcidr name so can use in cisco ios xr templates
+        #if node.ip.use_ipv6:
+            #ipv6_node = self.anm['ipv6'].node(node)
+            #node.loopback_zero.ipv6_address = ipv6_node.loopback
+            #node.loopback_zero.ipv6_subnet = node.loopback_subnet
+
         for interface in node.physical_interfaces:
             phy_int = self.anm['phy'].interface(interface)
             interface.physical = True
@@ -159,9 +179,12 @@ class RouterCompiler(DeviceCompiler):
 # TODO: for consistency, make ipv6_cidr
 
                     interface.ipv6_subnet = ipv6_int.subnet
-                    interface.ipv6_address = \
+                    try:
+                        interface.ipv6_address = \
                         sn_preflen_to_network(ipv6_int.ip_address,
                             interface.ipv6_subnet.prefixlen)
+                    except AttributeError:
+                        log.warning("Unable to format interface ")
 
         for interface in node.loopback_interfaces:
 
@@ -203,15 +226,53 @@ class RouterCompiler(DeviceCompiler):
 
         g_ospf = self.anm['ospf']
         g_ipv4 = self.anm['ipv4']
+        ospf_stanza = node.add_stanza("ospf")
 
         node.ospf.ipv4_mpls_te = False  # default, inherited enable if necessary
 
-        node.ospf.loopback_area = g_ospf.node(node).area
+        node.ospf.loopback_area = g_ospf.node(node).area or 0
 
         node.ospf.process_id = 1  # TODO: set this in build_network module
         node.ospf.lo_interface = self.lo_interface
 
         node.ospf.ospf_links = []
+
+        # aggregate by area
+        from collections import defaultdict
+        interfaces_by_area = defaultdict(list)
+
+        for interface in node.physical_interfaces:
+            if interface.exclude_igp:
+                continue  # don't configure IGP for this interface
+
+            ospf_int = g_ospf.interface(interface)
+            if ospf_int and ospf_int.is_bound:
+                area = ospf_int.area
+                #TODO: can we remove the next line?
+                area = str(area)  # can't serialize IPAddress object to JSON
+                #TODO: put in interface rather than interface.id for consistency
+                stanza = config_stanza(id = interface.id,
+                        cost = int(ospf_int.cost), passive = False)
+
+                if node.ip.use_ipv4:
+                    stanza.ipv4_address = ospf_int['ipv4'].ip_address
+                    stanza.ipv4_subnet = ospf_int['ipv4'].subnet
+                if node.ip.use_ipv6:
+                    stanza.ipv6_address = ospf_int['ipv6'].ip_address
+                    stanza.ipv6_subnet = ospf_int['ipv6'].subnet
+
+                interfaces_by_area[area].append(stanza)
+
+        loopback_zero = node.loopback_zero
+        ospf_loopback_zero = g_ospf.interface(loopback_zero)
+        router_area = ospf_loopback_zero.area  # area assigned to router
+        router_area = str(router_area)  # can't serialize IPAddress object to JSON
+        stanza = config_stanza(id = node.loopback_zero.id,
+            cost = 0, passive = True)
+        interfaces_by_area[router_area].append(stanza)
+
+        node.ospf.interfaces_by_area = config_stanza(**interfaces_by_area)
+
         added_networks = set()
         for interface in node.physical_interfaces:
             if interface.exclude_igp:
@@ -225,25 +286,27 @@ class RouterCompiler(DeviceCompiler):
             except TypeError:
                 try:
                     ospf_cost = netaddr.IPAddress(ospf_int.cost)
-                except TypeError:
+                except (TypeError, netaddr.AddrFormatError):
                     log.debug('Using default OSPF cost of 1 for %s on %s'
                                % (ospf_int, node))
                     ospf_cost = 1  # default
             interface.ospf_cost = ospf_cost
             network = ipv4_int.subnet
+
             if ospf_int and ospf_int.is_bound and network \
                 not in added_networks:  # don't add more than once
                 added_networks.add(network)
-                node.ospf.ospf_links.append(network=network,
-                        area=ospf_int.area)
+                link_stanza = config_stanza(network = network, interface = interface, area = ospf_int.area)
+                node.ospf.ospf_links.append(link_stanza)
 
             # also add networks for subnets to servers in the same AS
 
+
     def bgp(self, node):
         phy_node = self.anm['phy'].node(node)
-        g_bgp = self.anm['bgp']
         g_ipv4 = self.anm['ipv4']
-        g_ipv6 = self.anm['ipv6']
+        if node.ip.use_ipv6:
+            g_ipv6 = self.anm['ipv6']
         asn = phy_node.asn
         node.asn = asn
         node.bgp.ipv4_advertise_subnets = []
@@ -278,20 +341,21 @@ class RouterCompiler(DeviceCompiler):
 
         # TODO: check v6 hierarchy only created if node set to being v4 or v6
 
-        g_ibgp_v6 = self.anm['ibgp_v6']
-        for session in sort_sessions(g_ibgp_v6.edges(phy_node)):
-            if session.exclude:
-                log.debug('Skipping excluded ibgp session %s' % session)
-                continue  # exclude from regular ibgp config (eg VRF, VPLS, etc)
-            data = self.ibgp_session_data(session, ip_version=6)
+        if node.ip.use_ipv6:
+            g_ibgp_v6 = self.anm['ibgp_v6']
+            for session in sort_sessions(g_ibgp_v6.edges(phy_node)):
+                if session.exclude:
+                    log.debug('Skipping excluded ibgp session %s' % session)
+                    continue  # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                data = self.ibgp_session_data(session, ip_version=6)
 
-            direction = session.direction
-            if direction == 'down':
-                ibgp_rr_clients.append(data)
-            elif direction == 'up':
-                ibgp_rr_parents.append(data)
-            else:
-                ibgp_neighbors.append(data)
+                direction = session.direction
+                if direction == 'down':
+                    ibgp_rr_clients.append(data)
+                elif direction == 'up':
+                    ibgp_rr_parents.append(data)
+                else:
+                    ibgp_neighbors.append(data)
 
         # TODO: update this to use ibgp_v4 and ibgp_v6 overlays
 
@@ -310,13 +374,15 @@ class RouterCompiler(DeviceCompiler):
             data = self.ebgp_session_data(session, ip_version=4)
             ebgp_neighbors.append(data)
 
-        g_ebgp_v6 = self.anm['ebgp_v6']
-        for session in sort_sessions(g_ebgp_v6.edges(phy_node)):
-            if session.exclude:
-                log.debug('Skipping excluded ebgp session %s' % session)
-                continue  # exclude from regular ibgp config (eg VRF, VPLS, etc)
-            data = self.ebgp_session_data(session, ip_version=6)
-            ebgp_neighbors.append(data)
+        if node.ip.use_ipv6:
+            g_ebgp_v6 = self.anm['ebgp_v6']
+            for session in sort_sessions(g_ebgp_v6.edges(phy_node)):
+                if session.exclude:
+                    print "Exclude"
+                    log.debug('Skipping excluded ebgp session %s' % session)
+                    continue  # exclude from regular ibgp config (eg VRF, VPLS, etc)
+                data = self.ebgp_session_data(session, ip_version=6)
+                ebgp_neighbors.append(data)
 
         node.bgp.ebgp_neighbors = ebgp_neighbors
         node.bgp.ebgp_neighbors.sort('asn')
