@@ -1,16 +1,21 @@
+import itertools
 import os
 from datetime import datetime
+
 import autonetkit
 import autonetkit.config
 import autonetkit.log as log
 import autonetkit.plugins.naming as naming
 from autonetkit.ank import sn_preflen_to_network
+from autonetkit.ank_utils import call_log
+from autonetkit.compilers.device.cisco import (IosBaseCompiler,
+                                               IosClassicCompiler,
+                                               IosXrCompiler, NxOsCompiler,
+                                               StarOsCompiler)
 from autonetkit.compilers.platform.platform_base import PlatformCompiler
-import itertools
+from autonetkit.nidb import config_stanza
 
 
-from autonetkit.compilers.device.cisco import IosBaseCompiler, IosClassicCompiler, IosXrCompiler, NxOsCompiler, StarOsCompiler
-
 class CiscoCompiler(PlatformCompiler):
     """Platform compiler for Cisco"""
 
@@ -77,30 +82,49 @@ class CiscoCompiler(PlatformCompiler):
         for x in itertools.count(0):
             yield x
 
+    @call_log
     def compile(self):
+        self.copy_across_ip_addresses()
+        self.compile_devices()
+        self.assign_management_interfaces()
+
+    def _parameters(self):
+        g_phy = self.anm['phy']
         settings = autonetkit.config.settings
         to_memory = settings['Compiler']['Cisco']['to memory']
-#TODO: need to copy across the interface name from edge to the interface
-        g_phy = self.anm['phy']
         use_mgmt_interfaces = g_phy.data.mgmt_interfaces_enabled
-        if use_mgmt_interfaces:
-            log.info("Allocating management interfaces for Cisco")
-        else:
-            log.info("Not allocating management interfaces for Cisco")
 
-        log.info("Compiling Cisco for %s" % self.host)
+
         now = datetime.now()
         if settings['Compiler']['Cisco']['timestamp']:
             timestamp = now.strftime("%Y%m%d_%H%M%S_%f")
             dst_folder = os.path.join("rendered", self.host, timestamp, "cisco")
         else:
             dst_folder = os.path.join("rendered", self.host, "cisco")
+
+        #TODO: use a namedtuple
+        return to_memory, use_mgmt_interfaces, dst_folder
+
+    @call_log
+    def compile_devices(self):
+        g_phy = self.anm['phy']
+
+        to_memory, use_mgmt_interfaces, dst_folder = self._parameters()
+        if use_mgmt_interfaces:
+            log.info("Allocating VIRL management interfaces")
+        else:
+            log.info("Not allocating VIRL management interfaces")
+#TODO: need to copy across the interface name from edge to the interface
+
 # TODO: merge common router code, so end up with three loops: routers, ios
 # routers, ios_xr routers
 
     #TODO: Split out each device compiler into own function
 
+    #TODO: look for unused code paths here - especially for interface allocation
+
         # store autonetkit_cisco version
+        log.info("Generating device configurations")
         from pkg_resources import get_distribution
         ank_cisco_version = get_distribution("autonetkit_cisco").version
 
@@ -163,27 +187,9 @@ class CiscoCompiler(PlatformCompiler):
                     interface.id = phy_specified_id
 
                 #TODO: make this part of the base device compiler, which server/router inherits
-                if nidb_node.ip.use_ipv4:
-                    ipv4_int = phy_int['ipv4']
-                    if ipv4_int.is_bound:
-                        # interface is connected
-                        interface.use_ipv4 = True
-                        interface.ipv4_address = ipv4_int.ip_address
-                        interface.ipv4_subnet = ipv4_int.subnet
-                        interface.ipv4_cidr = sn_preflen_to_network(interface.ipv4_address,
-                                interface.ipv4_subnet.prefixlen)
-                if nidb_node.ip.use_ipv6:
-                    ipv6_int = phy_int['ipv6']
-                    if ipv6_int.is_bound:
-                        # interface is connected
-                        interface.use_ipv6 = True
-#TODO: for consistency, make ipv6_cidr
-                        interface.ipv6_subnet = ipv6_int.subnet
-                        interface.ipv6_address = sn_preflen_to_network(ipv6_int.ip_address,
-                                interface.ipv6_subnet.prefixlen)
 
             if use_mgmt_interfaces: # not these are physical interfaces; configure after previous config steps
-                mgmt_int = nidb_node.add_interface(management = True)
+                mgmt_int = nidb_node.add_interface(management = True, description = "eth0")
                 mgmt_int_id = "eth0"
                 mgmt_int.id = mgmt_int_id
 
@@ -294,6 +300,8 @@ class CiscoCompiler(PlatformCompiler):
                 if not interface.id:
                     interface.id = self.numeric_to_interface_label_nxos(interface.numeric_id)
 
+            nidb_node.supported_features = config_stanza(mpls_te = False, mpls_oam = False, vrf = False)
+
             nxos_compiler.compile(nidb_node)
             #TODO: make this work other way around
 
@@ -336,58 +344,17 @@ class CiscoCompiler(PlatformCompiler):
             nidb_node = self.nidb.node(phy_node)
             nidb_node.input_label = phy_node.id  # set specifically for now for other variants
 
-# TODO: use more os.path.join for render folders
-# TODO: Split compilers into seperate modules
-
-        if use_mgmt_interfaces:
-            self.assign_management_interfaces()
-
     def assign_management_interfaces(self):
         g_phy = self.anm['phy']
+        use_mgmt_interfaces = g_phy.data.mgmt_interfaces_enabled
+        if not use_mgmt_interfaces:
+            return
         lab_topology = self.nidb.topology[self.host]
         oob_management_ips = {}
 
-        #TODO: remove this code now allocated externally
-
-        #TODO: make this seperate function
-        from netaddr import IPNetwork, IPRange
-
-        mgmt_address_start = g_phy.data.mgmt_address_start
-        mgmt_address_end = g_phy.data.mgmt_address_end
-        mgmt_prefixlen = int(g_phy.data.mgmt_prefixlen)
-
-        #TODO: need to check if range is insufficient
-        mgmt_ips = (IPRange(mgmt_address_start, mgmt_address_end))
-        mgmt_ips_iter = iter(mgmt_ips) # to iterate over
-
-        mgmt_address_start_network = IPNetwork(mgmt_address_start) # as /32 for supernet
-        mgmt_address_end_network = IPNetwork(mgmt_address_end) # as /32 for supernet
-        # retrieve the first supernet, as this is the range requested. subsequent are the subnets
-        start_subnet = mgmt_address_start_network.supernet(mgmt_prefixlen)[0] # retrieve first
-        end_subnet = mgmt_address_end_network.supernet(mgmt_prefixlen)[0] # retrieve first
-
-        try: # validation
-            assert(start_subnet == end_subnet)
-            log.debug("Verified: Cisco management subnets match")
-        except AssertionError:
-            log.warning("Error: Cisco management subnets do not match: %s and %s, using start subnet"
-                    % (start_subnet, end_subnet))
-
-        mgmt_subnet = start_subnet
-        hosts_to_allocate = sorted(self.nidb.nodes('is_router', host=self.host))
+        hosts_to_allocate = sorted(self.nidb.nodes('is_l3device', host=self.host))
         dhcp_subtypes = {"vios"}
         dhcp_hosts = [h for h in hosts_to_allocate if h.device_subtype in dhcp_subtypes]
-        non_dhcp_hosts = [h for h in hosts_to_allocate if h.device_subtype not in dhcp_subtypes]
-
-        try: # validation
-            assert(len(mgmt_ips) >= len(non_dhcp_hosts))
-            log.debug("Verified: Cisco management IP range is sufficient size %s for %s hosts"
-                    % (len(mgmt_ips), len(non_dhcp_hosts)))
-        except AssertionError:
-            log.warning("Error: Cisco management IP range is insufficient size %s for %s hosts"
-                    % (len(mgmt_ips), len(non_dhcp_hosts)))
-            # TODO: need to use default range
-            return
 
         for nidb_node in hosts_to_allocate:
             for interface in nidb_node.physical_interfaces:
@@ -397,18 +364,8 @@ class CiscoCompiler(PlatformCompiler):
                     interface.mgmt = True
                     interface.comment = "Configured on launch"
                     if nidb_node.ip.use_ipv4:
-                        interface.use_ipv4 = False
+                        interface.use_ipv4 = False # want a "no ip address" stanza
                     if nidb_node.use_cdp:
                         interface.use_cdp = True # ensure CDP activated
                     if nidb_node in dhcp_hosts:
                         interface.use_dhcp = True
-                        oob_management_ips[str(nidb_node)] = "dhcp"
-                    else:
-                        ipv4_address = mgmt_ips_iter.next()
-                        interface.ipv4_address = ipv4_address
-                        interface.ipv4_subnet = mgmt_subnet
-                        interface.ipv4_cidr = sn_preflen_to_network(ipv4_address, mgmt_prefixlen)
-                        oob_management_ips[str(nidb_node)] = ipv4_address
-
-        lab_topology.oob_management_ips = oob_management_ips
-
