@@ -176,12 +176,12 @@ def main(options):
     if options.target == "cisco":
         # output target is Cisco
         log.info("Setting output target as Cisco")
-        settings['Graphml']['Node Defaults']['platform'] = "cisco"
+        settings['Graphml']['Node Defaults']['platform'] = "VIRL"
         settings['Graphml']['Node Defaults']['host'] = "internal"
         settings['Graphml']['Node Defaults']['syntax'] = "ios_xr"
         settings['Compiler']['Cisco']['to memory'] = 1
         settings['General']['deploy'] = 1
-        settings['Deploy Hosts']['internal'] = {'cisco':
+        settings['Deploy Hosts']['internal'] = {'VIRL':
                                                 {'deploy': 1}}
 
     if options.debug or settings['General']['debug']:
@@ -234,6 +234,8 @@ def main(options):
         log.error(
             "Error generating network configurations: %s. More information may be available in the debug log." % err)
         log.debug("Error generating network configurations", exc_info=True)
+        if settings['General']['stack_trace']:
+            print traceback.print_exc()
         import sys
         sys.exit("Unable to build configurations.")
 
@@ -278,17 +280,18 @@ def create_nidb(anm):
     g_ip = anm['ip']
     g_graphics = anm['graphics']
     nidb.add_nodes_from(
-        g_phy, retain=['label', 'host', 'platform', 'Network', 'update'])
+        g_phy, retain=['label', 'host', 'platform', 'Network', 'update', 'asn'])
 
     cd_nodes = [n for n in g_ip.nodes(
-        "collision_domain") if not n.is_switch]  # Only add created cds - otherwise overwrite host of switched
+        "broadcast_domain") if not n.is_switch]  # Only add created cds - otherwise overwrite host of switched
     nidb.add_nodes_from(
-        cd_nodes, retain=['label', 'host'], collision_domain=True)
+        cd_nodes, retain=['label', 'host'], broadcast_domain=True)
 
-    for node in nidb.nodes("collision_domain"):
+    for node in nidb.nodes("broadcast_domain"):
         ipv4_node = anm['ipv4'].node(node)
         if ipv4_node:
             node.ipv4_subnet = ipv4_node.subnet
+            #TODO: copy across IPv6 seperately
             node.ipv6_subnet = ipv4_node['ipv6'].subnet
 
 # add edges to switches
@@ -313,9 +316,13 @@ def compile_network(anm):
         if platform == "netkit":
             import autonetkit.compilers.platform.netkit as pl_netkit
             platform_compiler = pl_netkit.NetkitCompiler(nidb, anm, host)
-        elif platform == "cisco":
-            import autonetkit.compilers.platform.cisco as pl_cisco
-            platform_compiler = pl_cisco.CiscoCompiler(nidb, anm, host)
+        elif platform == "VIRL":
+            try:
+                import autonetkit_cisco.compilers.platform.cisco as pl_cisco
+                platform_compiler = pl_cisco.CiscoCompiler(nidb, anm, host)
+            except ImportError, e:
+                print e
+                log.warning("Unable to load VIRL platform compiler")
         elif platform == "dynagen":
             import autonetkit.compilers.platform.dynagen as pl_dynagen
             platform_compiler = pl_dynagen.DynagenCompiler(nidb, anm, host)
@@ -325,7 +332,7 @@ def compile_network(anm):
                 nidb, anm, host)
 
         if any(g_phy.nodes(host=host, platform=platform)):
-            log.info("Compile for %s on %s" % (platform, host))
+            log.info("Compiling configurations for %s on %s" % (platform, host))
             platform_compiler.compile()  # only compile if hosts set
         else:
             log.debug("No devices set for %s on %s" % (platform, host))
@@ -356,7 +363,7 @@ def deploy_network(anm, nidb, input_graph_string=None):
                     from autonetkit_cisco import deploy as cisco_deploy
                 except ImportError:
                     pass  # development module, may not be available
-                if platform == "cisco":
+                if platform == "VIRL":
                     create_new_xml = False
                     if not input_graph_string:
                         create_new_xml = True  # no input, eg if came from grid
@@ -382,7 +389,7 @@ def deploy_network(anm, nidb, input_graph_string=None):
                 netkit_deploy.extract(host, username, tar_file,
                                       config_path, timeout=60, key_filename=key_file,
                                       parallel_count=10)
-                if platform == "cisco":
+                if platform == "VIRL":
                     # TODO: check why using nklab here
                     cisco_deploy.package(config_path, "nklab")
 
