@@ -2,6 +2,9 @@ import autonetkit.ank as ank_utils
 import autonetkit.log as log
 
 
+from autonetkit.ank_utils import call_log
+
+@call_log
 def build_ibgp_v4(anm):
     #TODO: remove the bgp layer and have just ibgp and ebgp
     # TODO: build from design rules, currently just builds from ibgp links in bgp layer
@@ -11,9 +14,10 @@ def build_ibgp_v4(anm):
     g_ibgpv4 = anm.add_overlay("ibgp_v4", directed=True)
     ipv4_nodes = set(g_phy.nodes("is_router", "use_ipv4"))
     g_ibgpv4.add_nodes_from((n for n in g_bgp if n in ipv4_nodes),
-            retain = ["ibgp_level", "hrr_cluster", "rr_cluster"] )
+            retain = ["ibgp_level", "ibgp_role", "hrr_cluster", "rr_cluster"] )
     g_ibgpv4.add_edges_from(g_bgp.edges(type="ibgp"), retain="direction")
 
+@call_log
 def build_ibgp_v6(anm):
     #TODO: remove the bgp layer and have just ibgp and ebgp
     # TODO: build from design rules, currently just builds from ibgp links in bgp layer
@@ -23,9 +27,10 @@ def build_ibgp_v6(anm):
     g_ibgpv6 = anm.add_overlay("ibgp_v6", directed=True)
     ipv6_nodes = set(g_phy.nodes("is_router", "use_ipv6"))
     g_ibgpv6.add_nodes_from((n for n in g_bgp if n in ipv6_nodes),
-            retain = ["ibgp_level", "hrr_cluster", "rr_cluster"] )
+            retain = ["ibgp_level", "ibgp_role", "hrr_cluster", "rr_cluster"] )
     g_ibgpv6.add_edges_from(g_bgp.edges(type="ibgp"), retain="direction")
 
+@call_log
 def build_ebgp_v4(anm):
     #TODO: remove the bgp layer and have just ibgp and ebgp
     # TODO: build from design rules, currently just builds from ibgp links in bgp layer
@@ -46,6 +51,8 @@ def build_ebgp_v6(anm):
     g_ebgpv6.add_nodes_from(n for n in g_ebgp if n in ipv6_nodes)
     g_ebgpv6.add_edges_from(g_ebgp.edges(), retain="direction")
 
+
+@call_log
 def build_ebgp(anm):
     g_in = anm['input']
     g_phy = anm['phy']
@@ -57,14 +64,14 @@ def build_ebgp(anm):
     ebgp_switches = [n for n in g_in.nodes("is_switch")
             if not ank_utils.neigh_equal(g_phy, n, "asn")]
     g_ebgp.add_nodes_from(ebgp_switches, retain=['asn'])
-    log.debug("eBGP switches are %s" % ebgp_switches)
+    g_ebgp.log.debug("eBGP switches are %s" % ebgp_switches)
     g_ebgp.add_edges_from((e for e in g_in.edges()
             if e.src in ebgp_switches or e.dst in ebgp_switches),
     bidirectional=True, type='ebgp')
     ank_utils.aggregate_nodes(g_ebgp, ebgp_switches)
     # need to recalculate as may have aggregated
     ebgp_switches = list(g_ebgp.nodes("is_switch"))
-    log.debug("aggregated eBGP switches are %s" % ebgp_switches)
+    g_ebgp.log.debug("aggregated eBGP switches are %s" % ebgp_switches)
     exploded_edges = ank_utils.explode_nodes(g_ebgp, ebgp_switches)
     same_asn_edges = []
     for edge in exploded_edges:
@@ -77,14 +84,21 @@ def build_ebgp(anm):
     g_ebgp.remove_edges_from(same_asn_edges)
 
 
+@call_log
 def build_ibgp(anm):
     g_in = anm['input']
     g_phy = anm['phy']
     g_bgp = anm['bgp']
 
+
+    #TODO: build direct to ibgp graph - can construct combined bgp for vis
+
     ank_utils.copy_attr_from(g_in, g_bgp, "ibgp_level")
-    ank_utils.copy_attr_from(g_in, g_bgp, "ibgp_l2_cluster", "hrr_cluster")
-    ank_utils.copy_attr_from(g_in, g_bgp, "ibgp_l3_cluster", "rr_cluster")
+    ank_utils.copy_attr_from(g_in, g_bgp, "ibgp_l2_cluster", "hrr_cluster", default = None)
+    ank_utils.copy_attr_from(g_in, g_bgp, "ibgp_l3_cluster", "rr_cluster", default = None)
+
+    ank_utils.set_node_default(g_bgp,  is_hrr = None, is_rr = None, is_rrc=None)
+
 
     """Levels:
     0: no BGP
@@ -115,9 +129,17 @@ def build_ibgp(anm):
         elif n.ibgp_level == 3:
             n.is_rr = True
 
-    bgp_nodes = [n for n in g_bgp if not n.ibgp_level == 0]
+    ibgp_nodes = [n for n in g_bgp if not n.ibgp_level == 0]
 
-    for asn, asn_devices in ank_utils.groupby("asn", bgp_nodes):
+
+    # Notify user of non-ibgp nodes
+    non_ibgp_nodes = [n for n in g_bgp if n.ibgp_level == 0]
+    if len(non_ibgp_nodes) < 10:
+        log.info("Skipping iBGP for iBGP disabled nodes: %s" % non_ibgp_nodes)
+    elif len(non_ibgp_nodes) >= 10:
+        log.info("Skipping iBGP for iBGP disabled nodes: refer to visualization for resulting topology.")
+
+    for asn, asn_devices in ank_utils.groupby("asn", ibgp_nodes):
         asn_devices = list(asn_devices)
 
         asn_rrs = [n for n in asn_devices if n.is_rr]
@@ -155,7 +177,6 @@ def build_ibgp(anm):
             down_links = [(t, s) for (s, t) in up_links]
             g_bgp.add_edges_from(down_links, type='ibgp', direction='over')
 
-
             for hrr_cluster, hrr_cluster_rtrs in ank_utils.groupby("hrr_cluster", rr_cluster_rtrs):
                 hrr_cluster_rtrs = list(hrr_cluster_rtrs)
 
@@ -190,6 +211,47 @@ def build_ibgp(anm):
 
                     #TODO: Special case: if no hrr or rr cluster set, then connect to global RRs
 
+        #TODO: add consistency check to ensure is_hrr etc matches
+    """Levels:
+    0: no BGP
+    1: RRC
+    2: HRR
+    3: RR
+    """
+    levels_to_roles = {
+    0: "Disabled",
+    1: "RR Client",
+    2: "Hierarchical RR",
+    3: "Route Reflector",
+    }
+
+    expected_level_values = {
+    0: (True, None, None, None),
+    1: (None, True, None, None),
+    2: (None, None, True, None),
+    3: (None, None, None, True),
+    }
+
+    for node in g_bgp:
+        #TODO: move this to the validate stage
+        if node.top_level_peer:
+            continue # don't check the roles
+        values = (node.is_no_ibgp, node.is_rrc, node.is_hrr, node.is_rr)
+        if values.count(True) != 1:
+            log.warning("Inconsistency in iBGP attributes")
+        else:
+            if values != expected_level_values[node.ibgp_level]:
+                log.warning("Inconsistency in iBGP attributes")
+
+
+    for node in g_bgp:
+        if node.top_level_peer:
+            node.ibgp_role = "Top-level Peer"
+        else:
+            node.ibgp_role = levels_to_roles[node.ibgp_level]
+
+
+@call_log
 def build_bgp(anm):
     """Build iBGP end eBGP overlays"""
     # eBGP
