@@ -3,6 +3,7 @@ import functools
 import pprint
 import string
 import time
+from functools import total_ordering
 
 import ank_json
 import autonetkit.log as log
@@ -13,6 +14,56 @@ try:
 except ImportError:
     import pickle
 
+from collections import OrderedDict
+import logging
+
+
+class CustomAdapter(logging.LoggerAdapter):
+    def process(self, msg, kwargs):
+        return '[%s]: %s' % (self.extra['item'], msg), kwargs
+
+# based on http://docs.python.org/2.7/library/collections#collections.OrderedDict
+# and http://stackoverflow.com/q/455059
+class config_stanza(object):
+    def __init__(self, **kwargs):
+        object.__setattr__(self, '_odict', OrderedDict(kwargs))
+
+    def __repr__(self):
+        return str(self._odict.items())
+
+    def add_stanza(self, name, **kwargs):
+        """Adds a sub-stanza to this stanza"""
+        stanza = config_stanza(**kwargs)
+        self[name] = stanza
+        return stanza
+
+    def __setitem__(self, key, value):
+        if isinstance(value, dict):
+            log.warning("Adding dictionary %s: did you mean to add a config_stanza?" % key)
+        self._odict[key] = value
+
+    def __setattr__(self, key, value):
+        self._odict[key] = value
+
+    def __getattr__(self, key):
+        #TODO: decide how to return misses
+        try:
+            return self._odict[key]
+        except KeyError:
+            #TODO: log a key miss, and if strict turned on, give warning
+            return
+
+    def items(self):
+        #TODO map this to proper dict inherit to support these methods, keys, etc
+        return self._odict.items()
+
+    # TODO: add __iter__, __keys etc
+
+    #self.__dict__['_odict'][key] = value
+
+    #TODO: add a sort function that sorts the OrderedDict
+
+
 class interface_data_dict(collections.MutableMapping):
     """A dictionary which allows access as dict.key as well as dict['key']
     Based on http://stackoverflow.com/questions/3387691
@@ -168,6 +219,9 @@ class overlay_interface(object):
         object.__setattr__(self, 'nidb', nidb)
         object.__setattr__(self, 'node_id', node_id)
         object.__setattr__(self, 'interface_id', interface_id)
+        logger = logging.getLogger("ANK")
+        logstring = "Interface: %s" % str(self)
+        self.log = CustomAdapter(logger, {'item': logstring})
 
     def __key(self):
         # based on http://stackoverflow.com/q/2909106
@@ -319,6 +373,9 @@ class overlay_edge(object):
         object.__setattr__(self, 'nidb', nidb)
         object.__setattr__(self, 'src_id', src_id)
         object.__setattr__(self, 'dst_id', dst_id)
+        logger = logging.getLogger("ANK")
+        logstring = "Edge: %s" % str(self)
+        self.log = CustomAdapter(logger, {'item': logstring})
 
     def __repr__(self):
         return "(%s, %s)" % (self.src, self.dst)
@@ -384,6 +441,7 @@ class overlay_node_accessor(object):
         object.__setattr__(self, 'nidb', nidb)
         object.__setattr__(self, 'node_id', node_id)
 
+
     def __repr__(self):
         #TODO: make this list overlays the node is present in
         return "Overlay accessor for: %s" % self.nidb
@@ -501,6 +559,13 @@ class nidb_node(object):
 #Set using this method to bypass __setattr__
         object.__setattr__(self, 'nidb', nidb)
         object.__setattr__(self, 'node_id', node_id)
+        logger = logging.getLogger("ANK")
+        #TODO: also pass the node object to the logger for building custom output lists
+        # ie create a special handler that just outputs the specific node/link/interface errors
+        logstring = "Node: %s" % str(self)
+        self.log = CustomAdapter(logger, {'item': logstring})
+
+    #TODO: make a json objct that returns keys that aren't logs, etc - filter out
 
     def __repr__(self):
         return self._node_data['label']
@@ -511,6 +576,23 @@ class nidb_node(object):
     def __getstate__(self):
         return (self.nidb, self.node_id)
 
+    #TODO: add a dump method - needed with str()?
+
+    def add_stanza(self, name, **kwargs):
+        #TODO: decide if want shortcut for *args to set to True
+        if self.get(name):
+            value = self.get(name)
+            if isinstance(value, config_stanza):
+                # Don't recreate
+                self.log.debug("Stanza %s already exists" % name)
+                return value
+            else:
+                log.warning("Creating stanza: %s already set as %s for %s" % (name, type(value), self))
+
+        stanza = config_stanza(**kwargs)
+        self.__setattr__(name, stanza)
+        return stanza
+
     def __hash__(self):
         return hash(self.node_id)
 
@@ -628,8 +710,16 @@ class nidb_node(object):
 #TODO: make this work with ASN (which isn't always imported to NIDB)
         self_node_id = self.node_id
         other_node_id = other.node_id
-        self_node_string = [x for x in self.node_id if x not in string.digits]
-        other_node_string = [x for x in self.node_id if x not in string.digits]
+        try:
+            self_node_string = [x for x in self.node_id if x not in string.digits]
+        except TypeError:
+            self_node_string = self.node_id
+
+        try:
+            other_node_string = [x for x in other.node_id if x not in string.digits]
+        except TypeError:
+            other_node_string = other.node_id
+
         if self_node_string == other_node_string:
             self_node_id = "".join([x for x in self.node_id if x in string.digits])
             other_node_id = "".join([x for x in other.node_id if x in string.digits])
@@ -702,6 +792,10 @@ class nidb_node(object):
         except AttributeError:
             pass # not a dict
 
+        #TODO: remove once deprecated nidb_node_category
+        if isinstance(data, config_stanza):
+            return data
+
         try:
             data.keys()
             return nidb_node_category(self.nidb, self.node_id, key)
@@ -750,6 +844,7 @@ class nidb_graph_data(object):
 
 class lab_topology(object):
     """API to access lab topology in network"""
+    #TODO: replace this with config_stanza
 
     def __init__(self, nidb, topology_id):
 #Set using this method to bypass __setattr__
